@@ -3,9 +3,7 @@ import sys
 import os
 import argparse
 
-from Cython import Tempita as tempita
-
-# XXX: If this import ever fails (does it really?), vendor cython.tempita
+import tempita
 
 
 def process_tempita(fromfile, outfile=None):
