@@ -7,10 +7,10 @@ and that it appears to contain text prevalent for a NumPy binary
 distribution.
 
 """
-import os
 import sys
 import re
 import argparse
+import pathlib
 
 
 def check_text(text):
@@ -33,8 +33,12 @@ def main():
     __import__(args.module)
     mod = sys.modules[args.module]
 
+    # LICENSE.txt is installed in the .dist-info directory, so find it there
+    sitepkgs = pathlib.Path(mod.__file__).parent.parent
+    distinfo_path = list(sitepkgs.glob("numpy-*.dist-info"))[0]
+
     # Check license text
-    license_txt = os.path.join(os.path.dirname(mod.__file__), "LICENSE.txt")
+    license_txt = distinfo_path / "LICENSE.txt"
     with open(license_txt, encoding="utf-8") as f:
         text = f.read()
 
